"""Mariner - navigate torrents in CLI."""
VERSION = (1, 4, 1)

__title__ = "Mariner"
__description__ = "Navigate torrents in CLI"
__url__ = "http://gitlab.com/radek-sprta/mariner"
__download_url__ = "https://gitlab.com/radek-sprta/mariner/repository/archive.tar.gz?ref=master"
__version__ = ".".join(map(str, VERSION))
__author__ = "Radek Sprta"
__author_email__ = "mail@radeksprta.eu"
__license__ = "GNU GPLv3"
__copyright__ = "Copyright 2017 Radek Sprta"
